unit Cmap;

interface

type
  TColorMap = array[0..255,0..3] of integer;

const
  RANDOMCMAP = -1;
  NRCMAPS = 84;

procedure GetCmap(var Index: integer; const hue_rotation: double; out cmap: TColorMap);
procedure rgb2hsv(const rgb: array of double; out hsv: array of double);
procedure hsv2rgb(const hsv: array of double; out rgb: array of double);


implementation

uses
  cmapdata, Math;

procedure rgb2hsv(const rgb: array of double; out hsv: array of double);
var
  maxval, minval: double;
  del: double;
begin
  Maxval := Max(rgb[0], Max(rgb[1], rgb[2]));
  Minval := Min(rgb[0], Min(rgb[1], rgb[2]));

  hsv[2] := maxval; // v

  if (Maxval > 0) and (maxval <> minval) then begin
    del := maxval - minval;
    hsv[1] := del/Maxval;  //s

    hsv[0] := 0;
    if (rgb[0] > rgb[1]) and (rgb[0] > rgb[2]) then begin
      hsv[0] := (rgb[1] - rgb[2])/ del;
    end else if(rgb[1] > rgb[2]) then begin
      hsv[0] := 2 + (rgb[2] - rgb[0])/ del;
    end else begin
      hsv[0] := 4 + (rgb[0] - rgb[1])/ del;
    end;

    if hsv[0] < 0 then
      hsv[0] := hsv[0] + 6;

  end else begin
    hsv[0] := 0;
    hsv[1] := 0;
  end;
end;

procedure hsv2rgb(const hsv: array of double; out rgb: array of double);
var
  j: integer;
  f,p,q,t,v: double;
begin
  j := floor(hsv[0]);
  f := hsv[0] - j;
  v := hsv[2];
  p := hsv[2] * (1 - hsv[1]);
  q := hsv[2] * (1 - hsv[1] * f);
  t := hsv[2] * (1 - hsv[1] * (1 - f));

  case j of
  0: begin rgb[0] := v; rgb[1] := t; rgb[2] := p; end;
  1: begin rgb[0] := q; rgb[1] := v; rgb[2] := p; end;
  2: begin rgb[0] := p; rgb[1] := v; rgb[2] := t; end;
  3: begin rgb[0] := p; rgb[1] := q; rgb[2] := v; end;
  4: begin rgb[0] := t; rgb[1] := p; rgb[2] := v; end;
  5: begin rgb[0] := v; rgb[1] := p; rgb[2] := t; end;
  end;
end;

procedure GetCmap(var Index: integer; const hue_rotation: double; out cmap: TColorMap);
var
  i: Integer;
  rgb: array [0..2] of double;
  hsv: array [0..2] of double;
begin
  if Index = RANDOMCMAP then
    Index := Random(NRCMAPS);

  if (Index < 0) or (Index >= NRCMAPS) then
    Index := 0;

  for i := 0 to 255 do begin
    rgb[0] := cmaps[Index][i][0] / 255.0;
    rgb[1] := cmaps[Index][i][1] / 255.0;
    rgb[2] := cmaps[Index][i][2] / 255.0;

    rgb2hsv(rgb,hsv);
    hsv[0] := hsv[0] + hue_rotation * 6;
    hsv2rgb(hsv, rgb);

    cmap[i][0] := Round(rgb[0] * 255);
    cmap[i][1] := Round(rgb[1] * 255);
    cmap[i][2] := Round(rgb[2] * 255);
  end;
end;

end.

